<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\COZ\Services\Coz07200DistribuicaoDaRefeicaoService;

class Lista extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "lista";
    protected $distribuicaoRefeicaoService;
    protected $entityManager;
    protected $novoCodigo;


    public function __construct(
        $crudService, $parameterBag,
        Coz07200DistribuicaoDaRefeicaoService $distribuicaoRefeicaoService,
        $entityManager,
        $novoCodigo
    ) {
        $this->distribuicaoRefeicaoService = $distribuicaoRefeicaoService;
        $this->entityManager = $entityManager;
        $this->novoCodigo = $novoCodigo;
        parent::__construct($crudService, $parameterBag);
    }

    public function montaListaServConcat(DTO\Request $request, DTO\Response $response){
        $parametros = $request->getRow();
        $servicos = $parametros['SERVICO'];
        $grupserv = $parametros['GRUPOSERVICO'];
        $unidades = $parametros['UNIDADE'];
        $grupUnidade = $parametros['GRUPOUNIDADE'];
        $nrorg = $parametros['NRORG'];
        $central = $parametros['CENTRAL'];

        $lista = '';

        if($servicos) {
            // ok Parametro já vem concatenado com a unidade. (UNIDADE.SERVICO)
            if (is_array($servicos)){
                foreach ($servicos as $servico) {
                    $lista .= $servico['FILISERV'] . '_';
                }
            } else {
                $lista = $servicos.'_';
            }
        }
        else {
            $listaServico
                = $this->distribuicaoRefeicaoService->retornaServico(
                $nrorg, $central, $grupUnidade, $unidades, $grupserv
            );

            foreach ($listaServico as $servico) {
                $lista .= $servico['CDFILIAL'] . $servico['CDSERVFILI'] . '_';
            }
        }
        $lista = substr($lista, 0, -1);
        $response->addDataSet(new DataSet ('LISTASERVICO', array('LISTA' => $lista)));
    }
    
    public function geraLista($cdlista, $nrorg){
        $rows = array_unique(explode("_", $cdlista));
        
        $dataSourceName = 'lista';
        $nrlista = $this->novoCodigo->getCodeByTableName($dataSourceName, $nrorg);;
        $registros = array();
        foreach ($rows as $row) {
            if(!empty($row)){
                $row = array(
                    "NRLISTA"  => $nrlista,
                    "CDLISTA"  => $row,
                    "__is_new" => true,
                    "NRORG"    => $nrorg
                );
                array_push($registros, $row);
            }
        }
        
        $dataSet = new DataSet($dataSourceName, $registros);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
        $this->crudService->save($dataSourceName, $rowsToSave);
        
        return $nrlista;
    }
    
    public function insereListaProduto(DTO\Request $request, DTO\Response $response){
        $parametros = $request->getRow();
        $cdlista = $parametros['CDLISTA'];
        $nrorg = $parametros['NRORG'];
        $nrlista = $this->geraLista($cdlista, $nrorg);
        
        $response->addDataSet(new DataSet ('LISTAPRODUTO', array('LISTA' => $nrlista)));
    }

    public function insereLista(DTO\Request $request, DTO\Response $response){
        $parametros = $request->getRow();
        $cdLista = $parametros['CDLISTA'];
//        $cdLista = substr($parametros['CDLISTA'], 1);
        $delimitador = $parametros['DELIMITADOR'];
        $nrorg = $parametros['NRORG'];

        if ($delimitador) {
            if (substr($cdLista, -1) === $delimitador) {
                $cdLista = substr($cdLista, 0, -1);
            }

            /*$lista = explode($delimitador, $cdLista);
        } else {
            $lista = $cdLista;*/
        }

        $nrLista = $this->geraLista($cdLista, $nrorg);

        /*if ($lista[0] <> '') {
            $this->distribuicaoRefeicaoService->iniciaConexao();
            $nrLista = $this->distribuicaoRefeicaoService->novaLista($nrorg);

            foreach ($lista as $info) {
                $this->distribuicaoRefeicaoService->insereListaServico(
                    $nrorg, $nrLista, $info
                );
            }

            $this->distribuicaoRefeicaoService->realizaCommit();
        } else {
            $nrLista = '';
        }*/

        $response->addDataSet(
            new DataSet ('NRLISTA', array('LISTA' => $nrLista))
        );
    }
    
    public function insereListaServico(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $nrlistaServico = $this->geraListaServico($row['FILISERV'], $row['NRORG']);
        $response->addDataSet(
            new DataSet ('NRLISTASERVICO', array('NRLISTASERVICO' => $nrlistaServico))
        );
    }
    
    public function geraListaServico($filiserv, $nrorg){
        $servicosFiliais = explode("_",$filiserv);
        
        $nrlistaServico = $this->novoCodigo->getCodeByTableName('LISTA_SERVICO', $nrorg);
        
        $sql = "INSERT INTO LISTA_SERVICO (NRLISTA, CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, NRORG) VALUES (:NRLISTA, :CDFILIAL, :CDSERVFILI, :NMSERVICO, :CDGRUPSERV, :NRORG)";
        
        foreach ($servicosFiliais as $servicoFilial) {
            if(!empty($servicoFilial)){
                $servicoFilial = $this->infoServico($servicoFilial);
                if (count($servicoFilial) > 0) {
                    $stmt = $this->entityManager->getConnection()->prepare($sql);
                    $stmt->execute(
                        array(
                            "NRLISTA" => $nrlistaServico,
                            "CDFILIAL" => $servicoFilial['CDFILIAL'],
                            "NRORG" => $nrorg,
                            "CDSERVFILI" => $servicoFilial['CDSERVFILI'],
                            "NMSERVICO" => $servicoFilial['NMSERVICO'],
                            "CDGRUPSERV" => $servicoFilial['CDGRUPSERV']
                        )
                    );
                }
            }
            
        }
        
        return $nrlistaServico;
    }
    
    private function infoServico($row){
        $cdfilial = substr($row,0,4);
        $cdservfili = substr($row,4,5);
        $sql = "SELECT S.CDFILIAL, S.CDSERVFILI,S.NMSERVICO, G.CDGRUPSERV FROM SERVICO S
                    INNER JOIN GRUPSERV G ON G.CDGRUPSERV = S.CDGRUPSERV
                WHERE S.CDFILIAL =:CDFILIAL AND S.CDSERVFILI =:CDSERVFILI";
        $stmt = $this->entityManager->getConnection()->prepare($sql);
        $stmt->execute(
            array(
                "CDFILIAL"   => $cdfilial,
                "CDSERVFILI"  => $cdservfili
            )
        );
        $result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        if (count($result) == 0) return array();
        else return $result[0];
    }

    public function excluiLista(DTO\Request $request, DTO\Response $response) {
        $parametros = $request->getRow();

        $nrLista = $parametros['NRLISTA'];
        $nrorg = $parametros['NRORG'];

        $this->distribuicaoRefeicaoService->excluiLista($nrorg, $nrLista);
    }

    public function retornaPAnaoProd(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = "retornaPAnaoProd";
        $this->crudService->parameterBagColumns = array('NRORG', 'NRLISTA');

        $this->find($request, $response);
    }


}